/****************************************************************************
 * This simple module exports termprint function, which print a string on a 
 * terminal with busy waiting on device status
 *
 ****************************************************************************/


/****************************************************************************/
/* Inclusion of header files.                                               */
/****************************************************************************/

#include "h/const.h"
#include "h/types.h"

/****************************************************************************/
/* Inclusion of imported declarations.                                      */
/****************************************************************************/


/****************************************************************************/
/* Declarations strictly local to the module.                               */
/****************************************************************************/

#define TERM0BASE	0xB0000150
#define DEVLENGTH	16
#define WORDLENGTH	4
#define TRANSMCOMMAND	3
#define TRANSMSTATUS	2

#define READY	1
#define BUSY	3
#define TRANSMITTED	5

#define ACK	1
#define PRINTCHR	2
#define CHAROFFSET	8

#define STATUSMASK	0xFF

#define MAXTERM 	4

HIDDEN long unsigned int termstat(long unsigned int * stataddr);
 
/****************************************************************************/
/* Definitions to be exported.                                              */
/****************************************************************************/

/* This function prints a string on specified terminal and returns TRUE if 
 * print was successful, FALSE if not
 */
 
unsigned int termprint(char * str, unsigned int term)
{
	long unsigned int * statusp;
	long unsigned int * commandp;
	long unsigned int stat;
	long unsigned int cmd;
	unsigned int error = FALSE;
	
	if (term < MAXTERM)
	{
		/* terminal is correct */
		/* compute device register field addresses */
		statusp = (long unsigned int *) (TERM0BASE + (term * DEVLENGTH) + (TRANSMSTATUS * WORDLENGTH));
		commandp = (long unsigned int *) (TERM0BASE + (term * DEVLENGTH) + (TRANSMCOMMAND * WORDLENGTH));
		
		/* test device status */
		stat = termstat(statusp);
		if (stat == READY || stat == TRANSMITTED)
		{
			/* device is available */
			
			/* print cycle */
			while (*str != EOS && !error)		
			{
				cmd = (*str << CHAROFFSET) | PRINTCHR;
				*commandp = cmd;
				

				/* busy waiting */
				stat = termstat(statusp);
				while (stat == BUSY)
					 stat = termstat(statusp);
				
				/* end of wait */
				if (stat != TRANSMITTED)
					error = TRUE;
				else
					/* move to next char */
					str++;
			} 
		}
		else
			/* device is not available */
			error = TRUE;
	}
	else
		/* wrong terminal device number */
		error = TRUE;

	return (!error);		
}


/****************************************************************************/
/* Definitions strictly local to the module.                                */
/****************************************************************************/

/* This function returns the terminal transmitter status value given its address 
 */ 
HIDDEN long unsigned int termstat(long unsigned int * stataddr)
{
	return((*stataddr) & STATUSMASK);
}
