/* File: $Id: setup.e,v 1.19 1998/02/25 16:17:56 morsiani Exp $ */

/****************************************************************************
 *
 * External declarations from the corresponding module .cc
 *
 ****************************************************************************/


// This class implements SetupInfo objects. Each contains all configuration
// information for every other object in the program which may need it: see
// object contents.

class SetupInfo
{
	public:
		// This method builds a new SetupInfo object and set ROM file
		// configuration depending on boot type. If boot type is "core
		// boot", a kernel image must be loaded in RAM memory; for any other
		// kind of boot type, no kernel image needs to be loaded, but a
		// different Bootstrap ROM does.  Other system configuration
		// parameters are set to default, and devices are activated or
		// deactivated accordingly with file availability and default
		// configuration
		SetupInfo(unsigned int bType, const char * bootFile);

		// This method deletes all SetupInfo associated structures
		~SetupInfo(void);


		// the following methods allow to get the current configuration
		// values; their meaning and arguments are self-explanatory
		
		unsigned int getSpeed(void);
		Word getRamSize(void);
		const char * getROMFileName(unsigned int index);
		const char * DevIsActiveStr(unsigned int intl, unsigned int dnum);
		unsigned int getBootType(void);
		unsigned int getDevClass(unsigned int intl);
		const char * getDevClassStr(unsigned int intl);
		unsigned int getDevType(unsigned int intl, unsigned int dnum);
		const char * getDevTypeStr(unsigned int intl, unsigned int dnum);
		const char * getDevFileName(unsigned int intl, unsigned int dnum);
		Word getSymbolTableASID(void);
		
		
		// all these methods allow to update the current configuration values;
		// their meaning and arguments are almost self-explanatory
		
		void setSpeed(unsigned int val);
		void setRamSize(Word val);
		Boolean setROMFileName(unsigned int index, const char * fName);
		
		// This method detects if a device file for the specificated device
		// exists and is correct; if necessary it "activates" the device. 
		// It returns TRUE and set the file name to the new one if all is
		// OK, and returns FALSE (with no change in device file name or
		// status) otherwise.  If fname is NULL or empty, it simply sets the
		// device to "deactivated" and returns always TRUE.
		Boolean setDevFileName(unsigned int intl, unsigned int dnum, const char * fName);
		
		void setBootType(unsigned int bType);
		void setSymbolTableASID(Word asid);
		
	private:
	
		// processor speed in MHz
		unsigned int speed;
		
		// RAM size (in 4KB frames) 
		Word ramSize;

		// ASID associated to current symbol table		
		Word symTabASID;

		// boot type ("core", from disk, from tape)		
		unsigned int bootType;
		
		// device type current configuration 
		unsigned int devType[DEVINTUSED][DEVPERINT];
		
		// device file names
		char * devFName[DEVINTUSED][DEVPERINT];
		
		// ROM & symbol table file names 
		char * romFName[ROMSNUM];
		
		// This method tests device file validity. A device file is valid
		// when: if it is a tape or disk file, it contains the correct magic
		// number; if it is another kind of device, if file is writable
		// (since it is just a log file).  It returns TRUE or FALSE
		// according to file validity
		Boolean validFileMagic(Word tag, const char * fName);
};
